/**
   @file
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_GUIBASE_MULTI_SEQ_ITEM_H_INCLUDED
#define CNOID_GUIBASE_MULTI_SEQ_ITEM_H_INCLUDED

#include "Item.h"
#include <cnoid/MultiSeq>
#include "exportdecl.h"

namespace cnoid {

    class CNOID_EXPORT AbstractMultiSeqItem : public Item
    {
    public:
        AbstractMultiSeqItem();
        AbstractMultiSeqItem(const AbstractMultiSeqItem& org);
        virtual ~AbstractMultiSeqItem();

        virtual AbstractMultiSeqPtr abstractSeq() = 0;


    protected:
        virtual void doPutProperties(PutPropertyFunction& putProperty);
        virtual bool store(Archive& archive);
        virtual bool restore(const Archive& archive);

    };

    template <typename MultiSeqType>
    class MultiSeqItem : public AbstractMultiSeqItem
    {
    public:
        static void initialize(ExtensionManager* ext) { }
        
        typedef boost::intrusive_ptr< MultiSeqItem<MultiSeqType> > Ptr;
        
        MultiSeqItem() : seq_(new MultiSeqType()) { }
        
        MultiSeqItem(typename MultiSeqType::Ptr seq) : seq_(seq) { }

        virtual AbstractMultiSeqPtr abstractSeq() { return seq_; }

        typename MultiSeqType::Ptr seq() { return seq_; }

    protected:
            
        MultiSeqItem(const MultiSeqItem<MultiSeqType>& org)
            : AbstractMultiSeqItem(org),
              seq_(new MultiSeqType(*org.seq_)) { }

        /**
           For the copy constructor of inherited classes
        */
        MultiSeqItem(const MultiSeqItem<MultiSeqType>& org, typename MultiSeqType::Ptr newSeq)
            : AbstractMultiSeqItem(org),
              seq_(newSeq) { }

        
        virtual ~MultiSeqItem() { }

        void resetSeq(typename MultiSeqType::Ptr seq) { seq_ = seq; }
        
        virtual ItemPtr doDuplicate() const {
            return new MultiSeqItem<MultiSeqType>(*this);
        }

        virtual void doPutProperties(PutPropertyFunction& putProperty) {
            AbstractMultiSeqItem::doPutProperties(putProperty);
        }

        virtual bool store(Archive& archive) {
            return AbstractMultiSeqItem::store(archive);
        }
            
        virtual bool restore(const Archive& archive) {
            return AbstractMultiSeqItem::restore(archive);
        }

    private:
            
        typename MultiSeqType::Ptr seq_;
    };

}

#endif
